Name:           kasmvncserver
Version:        1.3.3
Release:        1%{?dist}
Summary:        VNC server accessible from a web browser

License:        GPLv2+
URL:            https://github.com/kasmtech/KasmVNC
Source0:        v1.3.3.tar.gz
Source1:        %{name}-noarch.tar.gz
Source2:        kasmvnc.yaml
Source3:        select-de.sh
Source4:        https://www.x.org/archive/individual/xserver/xorg-server-1.20.14.tar.gz

BuildRequires:  gcc g++ cmake
BuildRequires:  turbojpeg-devel
BuildRequires:  libxshmfence-devel
BuildRequires:  libwebp-devel
BuildRequires:  gnutls-devel
BuildRequires:  freetype-devel
BuildRequires:  openssl-devel
BuildRequires:  libpng-devel
BuildRequires:  libXcursor-devel
BuildRequires:  libXfixes-devel
BuildRequires:  libXrandr-devel
BuildRequires:  libXtst-devel
BuildRequires:  zlib-devel
BuildRequires:  libjpeg-devel
# Additional X11 dependencies
BuildRequires:  libXext-devel
BuildRequires:  libXrender-devel
BuildRequires:  libX11-devel
BuildRequires:  libXau-devel
# kasmxproxy dependencies
BuildRequires:  libXcursor-devel
BuildRequires:  libXrandr-devel
BuildRequires:  libXtst-devel
BuildRequires:  libXfixes-devel
BuildRequires:  libXext-devel
# Xorg server build dependencies
BuildRequires:  automake
BuildRequires:  autoconf
BuildRequires:  libtool
BuildRequires:  xorg-x11-util-macros
BuildRequires:  libdrm-devel
BuildRequires:  mesa-libGL-devel
BuildRequires:  pixman-devel
BuildRequires:  xorg-x11-proto-devel
BuildRequires:  libxkbfile-devel
BuildRequires:  libXfont2-devel
BuildRequires:  xkeyboard-config-devel
BuildRequires:  font-util
BuildRequires:  xorg-x11-xtrans-devel
BuildRequires:  mesa-libgbm-devel
BuildRequires:  flex
BuildRequires:  bison

Requires:       xorg-x11-xauth
Requires:       xkeyboard-config
Requires:       xkbcomp
Requires:       openssl
Requires:       perl
Requires:       perl-Switch
Requires:       perl-YAML-Tiny
Requires:       perl-Hash-Merge-Simple
Requires:       perl-Scalar-List-Utils
Requires:       perl-List-MoreUtils
Requires:       perl-Try-Tiny
Conflicts:      tigervnc-server
Conflicts:      tigervnc-server-minimal

%description
KasmVNC provides remote web-based access to a Desktop or application.
While VNC is in the name, KasmVNC differs from other VNC variants such
as TigerVNC, RealVNC, and TurboVNC. KasmVNC has broken from the RFB
specification which defines VNC, in order to support modern technologies
and increase security. KasmVNC is accessed by users from any modern
browser and does not support legacy VNC viewer applications. KasmVNC
uses a modern YAML based configuration at the server and user level,
allowing for ease of management. KasmVNC is maintained by Kasm
Technologies Corp, www.kasmweb.com.

WARNING: this package requires EPEL.

%prep
%setup -q -n KasmVNC-%{version}

%build
# Following the original build.sh flow, first build the base libraries
mkdir -p build
cd build
cmake -Wno-dev -DCMAKE_INSTALL_PREFIX=%{_builddir}/KasmVNC-%{version}/install_tmp \
    -DKASMVNC_OPTIMIZATIONS=ON \
    -DBUILD_VIEWER:BOOL=OFF \
    -DENABLE_GNUTLS:BOOL=OFF ..
make %{?_smp_mflags}

# Install the libraries
make install

# Build kasmxproxy
cd %{_builddir}/KasmVNC-%{version}/unix/kasmxproxy
gcc -o kasmxproxy kasmxproxy.c xxhash.c -I/usr/include/X11 -lX11 -lXext -lXtst -lXfixes -lXrandr -lXcursor

# Return to KasmVNC source root
cd %{_builddir}/KasmVNC-%{version}

# Start building Xvnc - follow build.sh flow
mkdir -p unix/xserver
tar -xf %{SOURCE4} -C unix/xserver --strip-components=1

cd unix/xserver
# Apply patches to Xorg server source
patch -Np1 -i ../xserver120.patch
patch -p0 < ../CVE-2022-2320-v1.20.patch

# Add subdir-objects option to configure.ac
sed -i 's/AM_INIT_AUTOMAKE\(\[.*\]\)/AM_INIT_AUTOMAKE([subdir-objects \1])/' configure.ac

# Regenerate configuration files
autoreconf -i

# Fix relative path references in Makefile.am before running configure
find hw -name "Makefile.am" -type f | xargs grep -l "libnetwork.la" | xargs sed -i "s|../../../../common/network/libnetwork.la|%{_builddir}/KasmVNC-%{version}/build/common/network/libnetwork.la|g"

# Set environment variables to ensure linking to correct libraries
export KASMVNC_SRCDIR="$(pwd)"
export LIBRARY_PATH="$(pwd)/unix/xserver/lib:$(pwd)/common:$(pwd)/unix/common:$(pwd)/unix/xserver/hw/vnc:$LIBRARY_PATH"
export LDFLAGS="-ljpeg -lz -lturbojpeg -L$(pwd)/unix/xserver/lib -L$(pwd)/common -L$(pwd)/unix/common -L$(pwd)/unix/xserver/hw/vnc $LDFLAGS"
export LD_LIBRARY_PATH="$(pwd)/unix/xserver/lib:$(pwd)/common:$(pwd)/unix/common:$(pwd)/unix/xserver/hw/vnc:$LD_LIBRARY_PATH"

# Create necessary directories and symlinks
mkdir -p %{_builddir}/KasmVNC-%{version}/common/rfb/.libs
mkdir -p %{_builddir}/KasmVNC-%{version}/common/rdr/.libs
mkdir -p %{_builddir}/KasmVNC-%{version}/common/os/.libs
mkdir -p %{_builddir}/KasmVNC-%{version}/common/network/.libs
mkdir -p %{_builddir}/KasmVNC-%{version}/common/Xregion/.libs
mkdir -p %{_builddir}/KasmVNC-%{version}/unix/common/.libs

# Create necessary .la file symlinks
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/rfb/librfb.la %{_builddir}/KasmVNC-%{version}/common/rfb/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/rdr/librdr.la %{_builddir}/KasmVNC-%{version}/common/rdr/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/os/libos.la %{_builddir}/KasmVNC-%{version}/common/os/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/network/libnetwork.la %{_builddir}/KasmVNC-%{version}/common/network/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/Xregion/libXregion.la %{_builddir}/KasmVNC-%{version}/common/Xregion/
ln -sf %{_builddir}/KasmVNC-%{version}/build/unix/common/libunixcommon.la %{_builddir}/KasmVNC-%{version}/unix/common/

# Create .a file symlinks in main directory
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/rfb/librfb.a %{_builddir}/KasmVNC-%{version}/common/rfb/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/rdr/librdr.a %{_builddir}/KasmVNC-%{version}/common/rdr/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/os/libos.a %{_builddir}/KasmVNC-%{version}/common/os/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/network/libnetwork.a %{_builddir}/KasmVNC-%{version}/common/network/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/Xregion/libXregion.a %{_builddir}/KasmVNC-%{version}/common/Xregion/
ln -sf %{_builddir}/KasmVNC-%{version}/build/unix/common/libunixcommon.a %{_builddir}/KasmVNC-%{version}/unix/common/

# Create .a file symlinks in .libs directory
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/rfb/.libs/librfb.a %{_builddir}/KasmVNC-%{version}/common/rfb/.libs/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/rdr/.libs/librdr.a %{_builddir}/KasmVNC-%{version}/common/rdr/.libs/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/os/.libs/libos.a %{_builddir}/KasmVNC-%{version}/common/os/.libs/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/network/.libs/libnetwork.a %{_builddir}/KasmVNC-%{version}/common/network/.libs/
ln -sf %{_builddir}/KasmVNC-%{version}/build/common/Xregion/.libs/libXregion.a %{_builddir}/KasmVNC-%{version}/common/Xregion/.libs/
ln -sf %{_builddir}/KasmVNC-%{version}/build/unix/common/.libs/libunixcommon.a %{_builddir}/KasmVNC-%{version}/unix/common/.libs/

# Configure Xorg server
./configure \
    --disable-config-hal \
    --disable-config-udev \
    --disable-dmx \
    --disable-dri \
    --disable-dri2 \
    --enable-dri3 \
    --enable-present \
    --disable-kdrive \
    --disable-static \
    --disable-xephyr \
    --disable-xinerama \
    --disable-xnest \
    --disable-xorg \
    --disable-xvfb \
    --disable-xwayland \
    --disable-xwin \
    --disable-selective-werror \
    --disable-dependency-tracking \
    --enable-glx \
    --prefix=/usr \
    --with-default-font-path="catalogue:/etc/X11/fontpath.d,built-ins" \
    --without-dtrace \
    --with-sha1=libcrypto \
    --with-xkb-bin-directory=/usr/bin \
    --with-xkb-output=/var/lib/xkb \
    --with-xkb-path=/usr/share/X11/xkb

# Remove array bounds error
find . -name "Makefile" -exec sed -i 's/-Werror=array-bounds//g' {} \;

# Compile Xvnc
make %{?_smp_mflags}

# Prepare final installation files
cd %{_builddir}/KasmVNC-%{version}
mkdir -p xorg.build/bin
cp -f unix/xserver/hw/vnc/Xvnc xorg.build/bin/ || cp -f `find unix/xserver -name Xvnc -type f | head -1` xorg.build/bin/
chmod +x xorg.build/bin/Xvnc

# Prepare man pages
mkdir -p xorg.build/man/man1
touch xorg.build/man/man1/Xserver.1
cp -f unix/xserver/hw/vnc/Xvnc.man xorg.build/man/man1/Xvnc.1 || touch xorg.build/man/man1/Xvnc.1

# Prepare DRI library links
mkdir -p xorg.build/lib
cd xorg.build/lib
if [ -d /usr/lib/x86_64-linux-gnu/dri ]; then
    ln -sfn /usr/lib/x86_64-linux-gnu/dri dri
elif [ -d /usr/lib/aarch64-linux-gnu/dri ]; then
    ln -sfn /usr/lib/aarch64-linux-gnu/dri dri
elif [ -d /usr/lib/arm-linux-gnueabihf/dri ]; then
    ln -sfn /usr/lib/arm-linux-gnueabihf/dri dri
elif [ -d /usr/lib/xorg/modules/dri ]; then
    ln -sfn /usr/lib/xorg/modules/dri dri
else
    ln -sfn /usr/lib64/dri dri
fi
cd %{_builddir}/KasmVNC-%{version}

# Build kasmvncpasswd
cd %{_builddir}/KasmVNC-%{version}/unix/kasmvncpasswd
mkdir -p build
cd build
cmake ..
make

%install
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT%{_bindir}
mkdir -p $RPM_BUILD_ROOT%{_mandir}/man1
mkdir -p $RPM_BUILD_ROOT%{_datadir}/kasmvnc/www
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/kasmvnc
mkdir -p $RPM_BUILD_ROOT%{_libdir}/kasmvncserver
mkdir -p $RPM_BUILD_ROOT%{_datadir}/doc/kasmvncserver
mkdir -p $RPM_BUILD_ROOT%{_datadir}/perl5

# Extract Web interface and related files first
tar -xf %{SOURCE1} -C $RPM_BUILD_ROOT

# Install binary files
cd %{_builddir}/KasmVNC-%{version}/build

# Install vncpasswd and kasmvncpasswd
cp -f unix/vncpasswd/vncpasswd $RPM_BUILD_ROOT%{_bindir}/
cp -f %{_builddir}/KasmVNC-%{version}/unix/kasmvncpasswd/build/kasmvncpasswd $RPM_BUILD_ROOT%{_bindir}/

# Install vncconfig
cp -f unix/vncconfig/vncconfig $RPM_BUILD_ROOT%{_bindir}/

# Install Xvnc
cp -f %{_builddir}/KasmVNC-%{version}/xorg.build/bin/Xvnc $RPM_BUILD_ROOT%{_bindir}/
chmod +x $RPM_BUILD_ROOT%{_bindir}/Xvnc

# Install perl modules and vncserver script
cp -a %{_builddir}/KasmVNC-%{version}/unix/KasmVNC $RPM_BUILD_ROOT%{_datadir}/perl5/
cp -f %{_builddir}/KasmVNC-%{version}/install_tmp/bin/vncserver $RPM_BUILD_ROOT%{_bindir}/

# Install kasmxproxy
cp -f %{_builddir}/KasmVNC-%{version}/unix/kasmxproxy/kasmxproxy $RPM_BUILD_ROOT%{_bindir}/
cp -f %{_builddir}/KasmVNC-%{version}/unix/kasmxproxy/kasmxproxy.man $RPM_BUILD_ROOT%{_mandir}/man1/kasmxproxy.1

# Install select-de.sh script
cp -f %{SOURCE3} $RPM_BUILD_ROOT%{_libdir}/kasmvncserver/

# Install man pages
cd %{_builddir}/KasmVNC-%{version}
cp -f install_tmp/share/man/man1/vncpasswd.1 $RPM_BUILD_ROOT%{_mandir}/man1/
cp -f install_tmp/share/man/man1/vncconfig.1 $RPM_BUILD_ROOT%{_mandir}/man1/
cp -f xorg.build/man/man1/Xvnc.1 $RPM_BUILD_ROOT%{_mandir}/man1/

# Create man page links
cd $RPM_BUILD_ROOT%{_mandir}/man1
ln -sf vncpasswd.1 kasmvncpasswd.1

# Copy documentation
cp -f %{_builddir}/KasmVNC-%{version}/LICENSE.TXT $RPM_BUILD_ROOT%{_datadir}/doc/kasmvncserver/
cp -f %{_builddir}/KasmVNC-%{version}/README.md $RPM_BUILD_ROOT%{_datadir}/doc/kasmvncserver/
cp -f %{_builddir}/KasmVNC-%{version}/BUILDING.txt $RPM_BUILD_ROOT%{_datadir}/doc/kasmvncserver/

# Install configuration file
cp -f %{SOURCE2} $RPM_BUILD_ROOT%{_sysconfdir}/kasmvnc/

# Modify certificate path in kasmvnc_defaults.yaml
sed -i -e 's!pem_certificate: .\+$!pem_certificate: /etc/pki/tls/private/kasmvnc.pem!' \
    $RPM_BUILD_ROOT%{_datadir}/kasmvnc/kasmvnc_defaults.yaml || true
sed -i -e 's!pem_key: .\+$!pem_key: /etc/pki/tls/private/kasmvnc.pem!' \
    $RPM_BUILD_ROOT%{_datadir}/kasmvnc/kasmvnc_defaults.yaml || true

# Modify stun_host in kasmvnc.yaml
sed -i 's/stun_host:.*/stun_host: ""/' $RPM_BUILD_ROOT%{_sysconfdir}/kasmvnc/kasmvnc.yaml

# Create symlink for compatibility with hardcoded path in vncserver script
mkdir -p $RPM_BUILD_ROOT/usr/lib
ln -sf %{_libdir}/kasmvncserver $RPM_BUILD_ROOT/usr/lib/kasmvncserver

%post
kasmvnc_group="kasmvnc-cert"

create_kasmvnc_group() {
  if ! getent group "$kasmvnc_group" >/dev/null; then
    groupadd --system "$kasmvnc_group"
  fi
}

make_self_signed_certificate() {
  local cert_file=/etc/pki/tls/private/kasmvnc.pem
  [ -f "$cert_file" ] && return 0

  openssl req -x509 -nodes -days 3650 -newkey rsa:2048 \
    -keyout "$cert_file" \
    -out "$cert_file" -subj \
    "/C=US/ST=VA/L=None/O=None/OU=DoFu/CN=kasm/emailAddress=none@none.none"
  chgrp "$kasmvnc_group" "$cert_file"
  chmod 640 "$cert_file"
}

create_kasmvnc_group
make_self_signed_certificate

%postun
rm -f /etc/pki/tls/private/kasmvnc.pem

%files
%defattr(-,root,root,-)
%config(noreplace) %{_sysconfdir}/kasmvnc
%{_bindir}/*
%{_libdir}/kasmvncserver
/usr/lib/kasmvncserver
%{_mandir}/man1/*
%{_datadir}/perl5/KasmVNC
%{_datadir}/kasmvnc
%license %{_datadir}/doc/kasmvncserver/LICENSE.TXT
%doc %{_datadir}/doc/kasmvncserver/README.md
%doc %{_datadir}/doc/kasmvncserver/BUILDING.txt

%changelog
* Thu Apr 25 2024 Fanjun Kong <fanjunkong@tencent.com> - 1.3.3-1
- Initial package for kasmvncserver
